/**
   @file    Context.cc
   @ingroup AnalysisAgent
   @brief   Execution context
   @verbatim
    Revision:       $Revision$
    Revision date:  $Date$
    Committed by:   $Author$

    This file is part of the Periscope performance measurement tool.
    See http://www.lrr.in.tum.de/periscope for details.

    Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
    See the COPYING file in the base directory of the package for details.
   @endverbatim
 */
#include "Context.h"
#include "psc_errmsg.h"

Context::Context( Region* reg_p, int rank_p, int thread_p ) {
    reg    = reg_p;
    rank   = rank_p;
    thread = thread_p;
}

Context::Context( int fileId_p, int rlf_p, int rank_p, int thread_p ) {
    reg = appl->searchRegion( fileId_p, rlf_p );
    if( !reg ) {
        psc_errmsg( "Region not found in context creation (%d,%d)!\n", fileId_p, rlf_p );
    }
    rank   = rank_p;
    thread = thread_p;
}

Context::~Context() {
}

Context* Context::copy() {
    Context* ct;

    ct = new Context( reg, rank, thread );
    return ct;
}

int Context::getFileId() {
    return reg->get_ident().file_id;
}

int Context::getRfl() {
    return reg->get_ident().rfl;
}
int Context::getStartPosition() {
    return reg->get_ident().start_position;
}

std::string Context::getFileName() {
    return reg->get_ident().file_name;
}

Region* Context::getRegion() {
    return reg;
}

RegionType Context::getRegionType() {
    return reg->get_ident().type;
}

int Context::getRank() {
    return rank;
}

int Context::getThread() {
    return thread;
}

std::string Context::getRegionId() {
    return reg->getRegionID();
}
