/**
   @file    AltixPBSStarter.h
   @ingroup Frontend
   @brief   PBS starter for Altix header
   @author	Michael Gerndt
   @verbatim
    Revision:       $Revision$
    Revision date:  $Date$
    Committed by:   $Author$

    This file is part of the Periscope performance measurement tool.
    See http://www.lrr.in.tum.de/periscope for details.

    Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
    See the COPYING file in the base directory of the package for details.
   @endverbatim
 */

#ifndef ALTIXPBSSTARTER_H_
#define ALTIXPBSSTARTER_H_

#include "Starter.h"
#include "string.h"
#include "BinStorage.h"
#include <list>


struct PBSAgentDetails {
    char             tag[ 100 ];
    char             host[ 100 ];
    char             relCPU[ 100 ];
    char             port[ 100 ];
    char             parent[ 100 ];
    char             applIds[ 4000 ];
    char             children[ 4000 ];
    PBSAgentDetails* nextAgent;
};

struct PBSLevelInfo {
    int                     numberOfAgents;
    struct PBSAgentDetails* agents;
    PBSLevelInfo*           nextLevel;
    PBSLevelInfo*           prevLevel;
};


/**
 * @class AltixPBSStarter
 * @ingroup Frontend
 *
 * @brief Starts the application in batch mode on HLRB2
 *
 * @sa Starter
 */
class AltixPBSStarter : public Starter {
    int idmap_len;
    int idmap_f[ 8192 ];
    int idmap_t[ 8192 ];
    int idmap_h_num[ 19 ];
    int idmap_h[ 19 ][ 8192 ];

    int         pbsnodelist_size;
    std::string pbsnodelist_string[ 8192 ];
    int         pbsnodelist_firstidx[ 8192 ];
    int         pbsnodelist_amount[ 8192 ];
    int         pbsnodelist_distributed[ 8192 ];
    BinStorage* stg;

//  int idmap_h_cpu[16][256];

public:
    AltixPBSStarter();

    virtual ~AltixPBSStarter();

    void constructPBSnodelist();

    bool runApplication();

    bool rerunApplication();

    bool runAgents();

    PBSLevelInfo* computeAgentHierarchy();

    void printAgentHierarchy( PBSLevelInfo* masterlevel );

    int getHostNumber( char* host );

    void runAnalysisAgent( PBSAgentDetails* agent );

    void runHlAgent( PBSAgentDetails* agent );
};

#endif /* ALTIXPBSSTARTER_H_ */
