/**
   @file	BGP_Starter.h
   @ingroup Frontend
   @brief   POE starter for IBM Power575 nodes header
   @author	Yury Oleynik
   @verbatim
        Revision:       $Revision$
        Revision date:  $Date$
        Committed by:   $Author$

        This file is part of the Periscope performance measurement tool.
        See http://www.lrr.in.tum.de/periscope for details.

        Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
        See the COPYING file in the base directory of the package for details.
   @endverbatim
 */

#ifndef BGP_STARTERSTARTER_H_
#define BGP_STARTERSTARTER_H_

#include "Starter.h"
#include <string>
#include <list>
/**
 * @class BGP_Starter
 * @ingroup Frontend
 *
 * @brief Starts the application on IBM Power575 nodes
 *
 * @sa Starter
 */
class BGP_Starter : public Starter {
    struct AgentDetails {
        char          tag[ 100 ];
        char          host[ 100 ];
        char          port[ 100 ];
        char          parent[ 100 ];
        char          applIds[ 4000 ];
        char          children[ 4000 ];
        AgentDetails* nextAgent;
    };

    struct LevelInfo {
        int                  numberOfAgents;
        struct AgentDetails* agents;
        LevelInfo*           nextLevel;
        LevelInfo*           prevLevel;
    };
    std::list <EntryData> processList;
    int                   idmap_len;
    int                   idmap_f[ 8192 ];
    int                   idmap_t[ 8192 ];
    int                   idmap_h_num[ 19 ];
    int                   idmap_h[ 19 ][ 8192 ];


private:
    LevelInfo* computeAgentHierarchy();

    void runHlAgent( AgentDetails* agent );

    void runAnalysisAgent( AgentDetails* agent );

    void printAgentHierarchy( LevelInfo* levels );

public:
    BGP_Starter();

    virtual ~BGP_Starter();

    bool runApplication();

    bool rerunApplication();

    bool runAgents();
};

#endif /* BGP_STARTERSTARTER_H_ */
