/*
 * BinStorage.h
 *
 *  Created on: 08.12.2008
 *      Author: Gerndt
 */

#ifndef BINSTORAGE_H_
#define BINSTORAGE_H_

#include <string>
#include <sstream>



struct cpuinfo_rec {
    bool assigned;
    bool free;
    int  binnum;
    int  relcpu;
    int  nearcpus_num;
    struct nearcpus_t {
        int num;
        int dist;
    } nearcpus[ 10000 ];
};



struct bininfo_rec {
    int  cpu_num;
    int  cpu[ 10000 ];
    int  nodeagent_cpu;
    bool active;
    bool cpu_active[ 10000 ];
};

/**
 * @class BinStorage
 * @ingroup Frontend
 *
 * @brief BinStorage Class???
 *
 * @sa Starter
 */
class BinStorage {
public:
    BinStorage();
    BinStorage( int numnodes,
                int cpuspernode );

    int read_cpuset( int node,
                     int nodeidx,
                     int id );

    void read_topology( int node,
                        int nodeidx,
                        int id );

    void output_current_bins( int node );

    void clear_data();

    void generate_bins( int node,
                        int max_cpus_per_bin );

    void activate_bins( int node,
                        int total_procs,
                        int ompnumthreads );

    void optimize_bins( int node );

    void optimize_bins_siman( int node );

    void optimize_bins_rand( int node );

    void create_placement_file( int   node,
                                char* placementFileName,
                                int   ompnumthreads,
                                int   total_procs );

    struct PBSLevelInfo* BinsToAgentHierarchy( int          numberOfNodes,
                                               std::string* pbsnodelist_string,
                                               int          ompnumthreads,
                                               int          total_procs,
                                               int*         idmap,
                                               int*         pbsnodelist_distributed );

private:
    int get_distance( int node,
                      int numcpu1,
                      int numcpu2 );

    int opt_get_measure( int node,
                         int bin_nr );

    int opt_nodeagent_get_position( int node,
                                    int bin_nr );

    bool opt_nodeagent_position( int node,
                                 int bin_nr );

    bool opt_nodeagent_position_siman( int   node,
                                       int   bin_nr,
                                       float pprob,
                                       float nprob );

    bool opt_pcpu_position( int node,
                            int src_bin_nr,
                            int dest_bin_nr );

    bool opt_pcpu_position_siman( int   node,
                                  int   src_bin_nr,
                                  int   dest_bin_nr,
                                  float pprob,
                                  float nprob );

    bool opt_pcpu_position_rand( int node,
                                 int src_bin_nr,
                                 int dest_bin_nr,
                                 int count );

    int          num_nodes;
    int          cpus_per_node;
    cpuinfo_rec* fullcpusetmap;
    int          bins_per_node;
    int*         fnum_of_cpubins;
    bininfo_rec* fullcpubins;
};

#endif /* BINSTORAGE_H_ */
