/**
   @file	FastInteractiveStarter.h
   @ingroup Frontend
   @brief   Starter for interactive jobs (optimized for launch speed)
   @author	Isaias Compres
   @verbatim
        Revision:       $Revision$
        Revision date:  $Date$
        Committed by:   $Author$

        This file is part of the Periscope performance measurement tool.
        See http://www.lrr.in.tum.de/periscope for details.

        Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
        See the COPYING file in the base directory of the package for details.
   @endverbatim
 */


#ifndef FASTINTERACTIVESTARTER_H_
#define FASTINTERACTIVESTARTER_H_

#include "Starter.h"
#include <string>

/**
 * @class FastInteractiveStarter
 * @ingroup Frontend
 *
 * @brief Starts the application in interactive mode (optimized for launch speed)
 *
 * @sa Starter
 */
class FastInteractiveStarter : public Starter {
    struct AgentDetails {
        char          tag[ 100 ];
        char          host[ 100 ];
        char          port[ 100 ];
        char          parent[ 100 ];
        char          applIds[ 4000 ];
        char          children[ 4000 ];
        AgentDetails* nextAgent;
    };

    struct LevelInfo {
        int                  numberOfAgents;
        struct AgentDetails* agents;
        LevelInfo*           nextLevel;
        LevelInfo*           prevLevel;
    };

    int                    idmap_len;
    int                    idmap_f[ 8192 ];
    int                    idmap_t[ 8192 ];
    int                    idmap_h_num[ 19 ];
    int                    idmap_h[ 19 ][ 8192 ];
    std::list< EntryData > processList;

private:
    LevelInfo* computeAgentHierarchy();

    void runHlAgent( AgentDetails* agent );

    void runAnalysisAgent( AgentDetails* agent );

    void printAgentHierarchy( LevelInfo* levels );

public:
    FastInteractiveStarter();

    virtual ~FastInteractiveStarter();

    bool runApplication();

    bool rerunApplication();

    bool runAgents();
};

#endif
