/**
   @file	InteractiveStarter.h
   @ingroup Frontend
   @brief   Starter for interactive jobs / Generic MPI starter header
   @author	Michael Gerndt
   @verbatim
        Revision:       $Revision$
        Revision date:  $Date$
        Committed by:   $Author$

        This file is part of the Periscope performance measurement tool.
        See http://www.lrr.in.tum.de/periscope for details.

        Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
        See the COPYING file in the base directory of the package for details.
   @endverbatim
 */


#ifndef INTERACTIVESTARTER_H_
#define INTERACTIVESTARTER_H_

#include "Starter.h"
#include <string>

/**
 * @class InteractiveStarter
 * @ingroup Frontend
 *
 * @brief Starts the application in interactive mode on HLRB2
 *
 * @sa Starter
 */
class InteractiveStarter : public Starter {
    struct AgentDetails {
        char          tag[ 100 ];
        char          host[ 100 ];
        char          port[ 100 ];
        char          parent[ 100 ];
        char          applIds[ 4000 ];
        char          children[ 4000 ];
        AgentDetails* nextAgent;
    };

    struct LevelInfo {
        int                  numberOfAgents;
        struct AgentDetails* agents;
        LevelInfo*           nextLevel;
        LevelInfo*           prevLevel;
    };

    int                    idmap_len;
    int                    idmap_f[ 8192 ];
    int                    idmap_t[ 8192 ];
    int                    idmap_h_num[ 19 ];
    int                    idmap_h[ 19 ][ 8192 ];
    std::list< EntryData > processList;

private:
    LevelInfo* computeAgentHierarchy();

    void runHlAgent( AgentDetails* agent );

    void runAnalysisAgent( AgentDetails* agent );

    void printAgentHierarchy( LevelInfo* levels );

public:
    InteractiveStarter();

    virtual ~InteractiveStarter();

    bool runApplication();

    bool rerunApplication();

    bool runAgents();
};

#endif /* INTERACTIVESTARTER_H_ */
