/*
 * BinStorage.cc
 *
 *  Created on: 08.12.2008
 *      Author: Gerndt
 */

#include "BinStorage.h"
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <sstream>
#include "AltixPBSStarter.h"
#include "psc_errmsg.h"
#include "frontend.h"

using namespace std;

BinStorage::BinStorage( int numnodes_,
                        int cpuspernode_ ) {
    num_nodes     = numnodes_;
    cpus_per_node = cpuspernode_;
    //psc_dbgmsg(1,"bin storage constructor with %d:%d\n",num_nodes,cpus_per_node);
    int ct;

    fullcpusetmap = ( cpuinfo_rec* )malloc( num_nodes * cpus_per_node * sizeof( cpuinfo_rec ) );
    if( fullcpusetmap == NULL ) {
        exit( 1 );
    }
    memset( fullcpusetmap, 0, num_nodes * cpus_per_node * sizeof( cpuinfo_rec ) );
    for( ct = 0; ct < num_nodes * cpus_per_node; ct++ ) {
        fullcpusetmap[ ct ].assigned = false;
        fullcpusetmap[ ct ].free     = false;
    }

    bins_per_node   = 512;
    fnum_of_cpubins = ( int* )malloc( num_nodes * sizeof( int ) );
    for( ct = 0; ct < num_nodes; ct++ ) {
        fnum_of_cpubins[ ct ] = 0;
    }

    fullcpubins = ( bininfo_rec* )malloc( num_nodes * bins_per_node * sizeof( bininfo_rec ) );
    if( fullcpubins == NULL ) {
        exit( 1 );
    }
    memset( fullcpubins, 0, num_nodes * bins_per_node * sizeof( bininfo_rec ) );

    srand( ( unsigned )time( 0 ) );
}

BinStorage::BinStorage() {
    BinStorage( 19, 512 );
}

void BinStorage::clear_data() {
    int ct;
    memset( fullcpusetmap, 0, num_nodes * cpus_per_node * sizeof( cpuinfo_rec ) );
    for( ct = 0; ct < num_nodes * cpus_per_node; ct++ ) {
        fullcpusetmap[ ct ].assigned = false;
        fullcpusetmap[ ct ].free     = false;
    }
    for( ct = 0; ct < num_nodes; ct++ ) {
        fnum_of_cpubins[ ct ] = 0;
    }
    memset( fullcpubins, 0, num_nodes * bins_per_node * sizeof( bininfo_rec ) );
}

void BinStorage::output_current_bins( int node ) {
    if( node >= num_nodes ) {
        return;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];
    for( int dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        psc_dbgmsg( 1, "node %d bin %d node agent %d\n", node, dct, pcpubins[ dct ].cpu[ 0 ] );
        for( int cct = 1; cct < pcpubins[ dct ].cpu_num; cct++ ) {
            psc_dbgmsg( 1, "  cpu %d\n", pcpubins[ dct ].cpu[ cct ] );
        }
    }
}

int BinStorage::read_cpuset( int node,
                             int nodeidx,
                             int id ) {
    if( node >= num_nodes ) {
        return -1;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];

    char tmpname[ 512 ];
    sprintf( tmpname, "%s/psctf3%i", getenv( "OPT_TMP" ), id );
    char sbuf[ 8192 ];

    //sprintf(sbuf,"/usr/pbs/default/bin/pbsdsh -v -n %i -- /usr/pbs/default/bin/pbs_attach -j $PBS_JOBID /bin/bash -c \'hostname; echo $$; NCPUSETE=`/bin/cat /proc/$$/cpuset`; echo $NCPUSETE; /bin/cat /dev/cpuset/$NCPUSETE/cpus; /bin/cat /dev/cpuset/$NCPUSETE/cpus > %s\'",nodeidx,tmpname);
    sprintf( sbuf, "/usr/pbs/default/bin/pbsdsh -n %i -- /usr/pbs/default/bin/pbs_attach -j $PBS_JOBID /bin/bash -c \'NCPUSETE=`/bin/cat /proc/$$/cpuset`; /bin/cat /dev/cpuset/$NCPUSETE/cpus > %s\'", nodeidx, tmpname );
    psc_dbgmsg( 5, "Command line for cpuset info: %s :\n", sbuf );
    int retVal = system( sbuf );


    int logical_cpu = 0;

    // reading out the cpuset data file
    FILE* pnfp = fopen( tmpname, "rt" );
    if( pnfp != NULL ) {
        char nfline[ 8192 ];
        if( fscanf( pnfp, "%s\n", nfline ) != EOF ) {
            //psc_dbgmsg(5,"cpus: >%s<\n",nfline);
            // tokeninze the string and put the IDs in the list
            char* elpos = strtok( nfline, ",\n" );
            while( elpos != 0 ) {
                char* rgpos = strchr( elpos, '-' );
                if( rgpos ) {
                    // element is a cpu range
//					psc_dbgmsg(1,"range >%s< ",elpos);
                    *rgpos = 0;
                    int cpu_from = atoi( elpos );
                    int cpu_to   = atoi( rgpos + 1 );
                    psc_dbgmsg( 5, "[%i - %i];\n", cpu_from, cpu_to );
                    for( int ct = cpu_from; ct <= cpu_to; ct++ ) {
                        cpusetmap[ ct ].assigned = true;
                        cpusetmap[ ct ].free     = true;
                        cpusetmap[ ct ].relcpu   = logical_cpu;
                        logical_cpu++;
                    }
                }
                else {
                    int cpunum = atoi( elpos );
                    if( cpunum <= 0 ) {
                        break;
                    }
                    psc_dbgmsg( 5, "cpu %i; ", cpunum );
                    cpusetmap[ cpunum ].assigned = true;
                    cpusetmap[ cpunum ].free     = true;
                    cpusetmap[ cpunum ].relcpu   = logical_cpu;
                    logical_cpu++;
                }
                // continue with next token
                elpos = strtok( 0, ",\n" );
            }
//			psc_dbgmsg(1,"\n");
        }
        fclose( pnfp );
    }
    else {
        psc_dbgmsg( 1, "Cannot open node info file %s\n", tmpname );
    }

    int cpus_in_set = 0;
    for( int ct = 0; ct < cpus_per_node; ct++ ) {
        if( cpusetmap[ ct ].assigned ) {
            cpus_in_set++;
        }
    }
    psc_dbgmsg( 5, "cpus in this set: %d\n", cpus_in_set );
    return cpus_in_set;
}

void BinStorage::read_topology( int node,
                                int nodeidx,
                                int id ) {
    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];

    int cpudistmap_tmp[ 8192 ];

    char tmpname[ 512 ];
    sprintf( tmpname, "%s/psctf3%i", getenv( "OPT_TMP" ), id );
    char sbuf[ 8192 ];
    sprintf( sbuf, "/usr/pbs/default/bin/pbsdsh -n %i -- /usr/pbs/default/bin/pbs_attach -j $PBS_JOBID /bin/bash -c \'/bin/cat /proc/sgi_sn/sn_topology > %s\'", nodeidx, tmpname );
    int retVal = system( sbuf );


    // read out the topology information
    FILE* pnfp = fopen( tmpname, "rt" );
    if( pnfp != NULL ) {
        char nfline[ 8192 ];
        while( fgets( nfline, 8190, pnfp ) != NULL ) {
            char oline[ 512 ];
            char otmp[ 512 ];
            sprintf( oline, "tokenized: " );
            // tokeninze the string and put the IDs in the list
            char* elpos = strtok( nfline, " \n" );
            if( elpos == 0 ) {
                continue;
            }
            if( !strcmp( elpos, "cpu" ) ) {
                elpos = strtok( 0, " \n" );
                if( elpos == 0 ) {
                    break;
                }
                int cpunum = atoi( elpos );
                sprintf( otmp, " %i", cpunum );
                strcat( oline, otmp );

                elpos = strtok( 0, ",\n" );
                if( elpos == 0 ) {
                    continue;
                }
                elpos = strtok( 0, ",\n" );
                if( elpos == 0 ) {
                    continue;
                }

                elpos = strtok( 0, " \n" );
                if( elpos == 0 ) {
                    continue;
                }
                if( !strcmp( elpos, "dist" ) ) {
                    strcat( oline, " withdist: " );
                    elpos = strtok( 0, ":\n" );
                    int dcpunum = 0;
                    int ct;
                    for( ct = 0; ct < cpus_per_node; ct++ ) {
                        cpudistmap_tmp[ ct ] = 0;
                    }
                    while( elpos != 0 ) {
                        int cpudist = atoi( elpos );
                        if( dcpunum != cpunum ) {
                            if( cpusetmap[ dcpunum ].assigned ) {
                                cpudistmap_tmp[ dcpunum ] = cpudist;
                            }
                        }
                        dcpunum++;
                        elpos = strtok( 0, ":\n" );
                    }

                    int min;
                    for(;; ) {
                        min = 0;
                        // get the minimal distance from this cpu to another allocated (!)
                        // and not yet treated cpu
                        for( ct = 0; ct < cpus_per_node; ct++ ) {
                            if( cpudistmap_tmp[ ct ] ) {
                                if( ( min == 0 ) || ( cpudistmap_tmp[ ct ] < min ) ) {
                                    min = cpudistmap_tmp[ ct ];
                                }
                            }
                        }
                        if( min == 0 ) {
                            break;
                        }
                        int ncpus_added = 0;
                        for( ct = 0; ct < cpus_per_node; ct++ ) {
                            if( cpudistmap_tmp[ ct ] == min ) {
                                // add the cpu at ct to the nearcpus list
                                int ncn = cpusetmap[ cpunum ].nearcpus_num;
                                cpusetmap[ cpunum ].nearcpus[ ncn ].num  = ct;
                                cpusetmap[ cpunum ].nearcpus[ ncn ].dist = min;
                                cpudistmap_tmp[ ct ]                     = 0;
                                ncpus_added++;
                                cpusetmap[ cpunum ].nearcpus_num = ncn + 1;
                                if( ncn >= cpus_per_node - 1 ) {
                                    break;
                                }
                            }
                        }
//						if (cpusetmap[cpunum].nearcpus_num>=16) break;
                        if( cpusetmap[ cpunum ].nearcpus_num >= cpus_per_node ) {
                            break;
                        }
                        if( ncpus_added == 0 ) {
                            break;
                        }
                    }
//                    psc_dbgmsg( 1, "csm >%s<\n", oline );
                }
            }
        }
        fclose( pnfp );
    }
    else {
        psc_dbgmsg( 1, "Cannot open topology info file %s\n", tmpname );
    }
}


void BinStorage::generate_bins( int node,
                                int max_cpus_per_bin ) {
    fnum_of_cpubins[ node ] = 0;
    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
    bininfo_rec* pcpubins  = &fullcpubins[ node * bins_per_node ];

    int num_cpubins = 0;
    for(;; ) {
        bool found = false;
        int  ccpu  = -1;
        // looking for a free cpu to host the node level agent
        for(; ccpu < cpus_per_node; ccpu++ ) {
            if( ( cpusetmap[ ccpu ].assigned ) && ( cpusetmap[ ccpu ].free ) ) {
                cpusetmap[ ccpu ].free = false;
                found                  = true;
                break;
            }
        }
        if( found ) {
            //psc_dbgmsg(1,"bin %d, first cpu: %d\n",num_cpubins,ccpu);
        }
        else {
            psc_dbgmsg( 5, "Finished allocating bins. Total bins: %d\n", num_cpubins );
            break;
        }

        // assign to bin
        pcpubins[ num_cpubins ].active = false;
        int ncpub = pcpubins[ num_cpubins ].cpu_num;
        pcpubins[ num_cpubins ].cpu[ ncpub ]        = ccpu;
        pcpubins[ num_cpubins ].cpu_active[ ncpub ] = false;
        ncpub++;
        pcpubins[ num_cpubins ].cpu_num = ncpub;

        // search close cpus to host application components
        for( int ncpuct = 0; ncpuct < cpusetmap[ ccpu ].nearcpus_num; ncpuct++ ) {
            int ncpunum = cpusetmap[ ccpu ].nearcpus[ ncpuct ].num;
            if( cpusetmap[ ncpunum ].free ) {
                pcpubins[ num_cpubins ].cpu[ ncpub ]        = ncpunum;
                pcpubins[ num_cpubins ].cpu_active[ ncpub ] = false;
                ncpub++;
                psc_dbgmsg( 5, "  found free: %d, now %d. cpu in bin\n", ncpunum, ncpub );
                pcpubins[ num_cpubins ].cpu_num = ncpub;
                cpusetmap[ ncpunum ].free       = false;
                if( ncpub >= max_cpus_per_bin + 1 ) {
                    break;
                }
            }
        }

        num_cpubins++;
    }

    fnum_of_cpubins[ node ] = num_cpubins;

    for( int ct = 0; ct < num_cpubins; ct++ ) {
        psc_dbgmsg( 1, "cpus for node %d in bin %d: %d\n", node, ct, pcpubins[ ct ].cpu_num );
    }
}

void BinStorage::activate_bins( int node,
                                int total_procs,
                                int ompnumthreads ) {
//Determines those cluster that are actually needed. If more clusters are
//available, the other clusters are marked inactive.
//total_procs is the number of processors to be used on that node

    if( node >= num_nodes ) {
        return;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];

    int cbin_d  = 0;
    int cproc_d = 1;    // skip analysisagent cpu
    for( int dct = 0; dct < total_procs; dct = dct + ompnumthreads ) {
        psc_dbgmsg( 5, "%d. MPI process in bin %d cpu %d\n", dct / ompnumthreads, cbin_d, cproc_d );
        // see if this bin is large enough (nodeagent+ompthreads)
        while( pcpubins[ cbin_d ].cpu_num < 1 + ompnumthreads ) {
            //current bin too small to hold a process with its threads
            //goto next bin
            cbin_d++;
            cproc_d = 1;
            if( cbin_d > fnum_of_cpubins[ node ] ) {
                psc_dbgmsg( 1, "not enough well-sized bins available!\n" );
                exit( 1 );
                break;
            }
        }
        pcpubins[ cbin_d ].active = true;
        cproc_d                  += ompnumthreads;      // add ompthreads
        if( cproc_d >= pcpubins[ cbin_d ].cpu_num ) {
            cbin_d++;
            cproc_d = 1;                // skip nodeagent cpu
        }
    }
}

// get the distance value of two cpus through the nearcpu list of the first cpu
int BinStorage::get_distance( int node,
                              int numcpu1,
                              int numcpu2 ) {
    if( node >= num_nodes ) {
        return -1;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];

    int ncn        = cpusetmap[ numcpu1 ].nearcpus_num;
    int found_dist = -1;
    for( int nct = 0; nct < ncn; nct++ ) {
        int nearcpu = cpusetmap[ numcpu1 ].nearcpus[ nct ].num;
        if( nearcpu == numcpu2 ) {
            found_dist = cpusetmap[ numcpu1 ].nearcpus[ nct ].dist;
            break;
        }
    }
    return found_dist;
}

int BinStorage::opt_get_measure( int node,
                                 int bin_nr ) {
    if( node >= num_nodes ) {
        return 0;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];
    if( !pcpubins[ bin_nr ].active ) {
        return 0;
    }

    // current total measure for the bin
    int tdist_s = 0;
    for( int cpuc = 1; cpuc < pcpubins[ bin_nr ].cpu_num; cpuc++ ) {
        int ncpu_m     = pcpubins[ bin_nr ].cpu[ 0 ];
        int ccpu_m     = pcpubins[ bin_nr ].cpu[ cpuc ];
        int found_dist = get_distance( node, ncpu_m, ccpu_m );

        if( found_dist >= 0 ) {
            tdist_s += found_dist;
//			psc_dbgmsg(1,"distance information for %d--%d: %d [sum %d]\n",ncpu_m,ccpu_m,found_dist,tdist_s);
        }
        else {
            psc_dbgmsg( 5, "couldn't find distance information for %d--%d\n", ncpu_m, ccpu_m );
        }
    }
    return tdist_s;
}

int BinStorage::opt_nodeagent_get_position( int node,
                                            int bin_nr ) {
    if( node >= num_nodes ) {
        return 0;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];
    if( !pcpubins[ bin_nr ].active ) {
        return 0;
    }

    int min_sum    = 0;
    int min_sum_na = 0;
    // test all node agent placements
    for( int ncpuc = 0; ncpuc < pcpubins[ bin_nr ].cpu_num; ncpuc++ ) {
        int nccpu_m = pcpubins[ bin_nr ].cpu[ ncpuc ];
        int tdist_s = 0;

        for( int cpuc = 0; cpuc < pcpubins[ bin_nr ].cpu_num; cpuc++ ) {
            if( ncpuc == cpuc ) {
                continue;                               // exclude the currently tested node level agent cpu
            }
            int found_dist = get_distance( node, nccpu_m, pcpubins[ bin_nr ].cpu[ cpuc ] );
            if( found_dist >= 0 ) {
                tdist_s += found_dist;
            }
        }

        if( ncpuc == 0 ) {
            min_sum = tdist_s;
        }
        else {
            if( min_sum > tdist_s ) {
                // having the node agent on cpu ncpuc is an improvement, save ncpuc
                min_sum    = tdist_s;
                min_sum_na = ncpuc;
            }
        }
//		psc_dbgmsg(1,"at cpu %d, minsum=%d, minat=%d\n",ncpuc,min_sum,min_sum_na);
    }

    if( min_sum_na > 0 ) {
        psc_dbgmsg( 5, "new nodeagent: %d [sum:%d]\n", min_sum_na, min_sum );
    }
    return min_sum_na;
}

bool BinStorage::opt_nodeagent_position( int node,
                                         int bin_nr ) {
    if( node >= num_nodes ) {
        return false;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];
    if( !pcpubins[ bin_nr ].active ) {
        return false;
    }

    int new_na_pos = opt_nodeagent_get_position( node, bin_nr );
    if( new_na_pos > 0 ) {
        int tmpc = pcpubins[ bin_nr ].cpu[ 0 ];
        pcpubins[ bin_nr ].cpu[ 0 ]          = pcpubins[ bin_nr ].cpu[ new_na_pos ];
        pcpubins[ bin_nr ].cpu[ new_na_pos ] = tmpc;
        return true;
    }
    return false;
}

bool BinStorage::opt_nodeagent_position_siman( int   node,
                                               int   bin_nr,
                                               float pprob,
                                               float nprob ) {
    if( node >= num_nodes ) {
        return false;
    }
    bininfo_rec* pcpubins = &fullcpubins[ node * bins_per_node ];
    if( !pcpubins[ bin_nr ].active ) {
        return false;
    }
    int min_sum    = 0;
    int min_sum_na = 0;
    // test all node agent placements
    for( int ncpuc = 0; ncpuc < pcpubins[ bin_nr ].cpu_num; ncpuc++ ) {
        int nccpu_m = pcpubins[ bin_nr ].cpu[ ncpuc ];
        int tdist_s = 0;

        for( int cpuc = 0; cpuc < pcpubins[ bin_nr ].cpu_num; cpuc++ ) {
            if( ncpuc == cpuc ) {
                continue;                               // exclude the currently tested node level agent cpu
            }
            int found_dist = get_distance( node, nccpu_m, pcpubins[ bin_nr ].cpu[ cpuc ] );
            if( found_dist >= 0 ) {
                tdist_s += found_dist;
            }
        }

        if( ncpuc == 0 ) {
            min_sum = tdist_s;
        }
        else {
            bool newna = false;
            if( min_sum > tdist_s ) {
                // having the node agent on cpu ncpuc is an improvement
                if( ( float )rand() / RAND_MAX < pprob ) {
                    newna = true;
                }
            }
            else {
                if( ( float )rand() / RAND_MAX < nprob ) {
                    newna = true;
                }
            }
            if( newna ) {
                min_sum    = tdist_s;
                min_sum_na = ncpuc;
            }
        }
//		psc_dbgmsg(1,"at cpu %d, minsum=%d, minat=%d\n",ncpuc,min_sum,min_sum_na);
    }

    if( min_sum_na > 0 ) {
        psc_dbgmsg( 5, "new nodeagent: %d [sum:%d]\n", min_sum_na, min_sum );
        int tmpc = pcpubins[ bin_nr ].cpu[ 0 ];
        pcpubins[ bin_nr ].cpu[ 0 ]          = pcpubins[ bin_nr ].cpu[ min_sum_na ];
        pcpubins[ bin_nr ].cpu[ min_sum_na ] = tmpc;
        return true;
    }
    return false;
}

bool BinStorage::opt_pcpu_position( int node,
                                    int src_bin_nr,
                                    int dest_bin_nr ) {
    if( node >= num_nodes ) {
        return false;
    }
    bininfo_rec* pcpubins       = &fullcpubins[ node * bins_per_node ];
    bool         swap_performed = false;
    for( int dcp = 1; dcp < pcpubins[ src_bin_nr ].cpu_num; dcp++ ) {
        // get the distance information from the cpu at dcp to its node level agent cpu
        int dcp_dist = get_distance( node, pcpubins[ src_bin_nr ].cpu[ 0 ], pcpubins[ src_bin_nr ].cpu[ dcp ] );
        if( dcp_dist < 0 ) {
            continue;
        }

        for( int tcp = 1; tcp < pcpubins[ dest_bin_nr ].cpu_num; tcp++ ) {
            // get the distance information from the cpu at tcp to its node level agent cpu
            int tcp_dist = get_distance( node, pcpubins[ dest_bin_nr ].cpu[ 0 ], pcpubins[ dest_bin_nr ].cpu[ tcp ] );
            if( tcp_dist < 0 ) {
                continue;
            }

            // source bin: src_bin_nr,  source cpu: dcp,  dist to nodecpu: dcp_dist
            // dest bin: dest_bin_nr,   dest cpu: tcp,    dist to nodecpu: tcp_dist

            int dcp_dist_new = get_distance( node, pcpubins[ src_bin_nr ].cpu[ 0 ], pcpubins[ dest_bin_nr ].cpu[ tcp ] );
            int tcp_dist_new = get_distance( node, pcpubins[ dest_bin_nr ].cpu[ 0 ], pcpubins[ src_bin_nr ].cpu[ dcp ] );

            if( dcp_dist_new < 0 ) {
                continue;
            }
            if( tcp_dist_new < 0 ) {
                continue;
            }

            // see if the swapping would improve the measure
            if( ( ( pcpubins[ dest_bin_nr ].active ) && ( dcp_dist + tcp_dist > dcp_dist_new + tcp_dist_new ) ) ||
                ( ( !pcpubins[ dest_bin_nr ].active ) && ( dcp_dist > dcp_dist_new ) ) ) {
/*                psc_dbgmsg(1,"swapping %d:%d:%d(%d->%d) with [active: %x] %d:%d:%d(%d->%d)\n",
                                        src_bin_nr,dcp,pcpubins[src_bin_nr].cpu[dcp],dcp_dist,dcp_dist_new,
                                        pcpubins[dest_bin_nr].active,
                                        dest_bin_nr,tcp,pcpubins[dest_bin_nr].cpu[tcp],tcp_dist,tcp_dist_new); */
                int tmpcpu = pcpubins[ src_bin_nr ].cpu[ dcp ];
                pcpubins[ src_bin_nr ].cpu[ dcp ]  = pcpubins[ dest_bin_nr ].cpu[ tcp ];
                pcpubins[ dest_bin_nr ].cpu[ tcp ] = tmpcpu;
                swap_performed                     = true;
            }
        } // end for tcp
    }     // end for dcp
    return swap_performed;
}

bool BinStorage::opt_pcpu_position_siman( int   node,
                                          int   src_bin_nr,
                                          int   dest_bin_nr,
                                          float pprob,
                                          float nprob ) {
    if( node >= num_nodes ) {
        return false;
    }
    bininfo_rec* pcpubins       = &fullcpubins[ node * bins_per_node ];
    bool         swap_performed = false;
    for( int dcp = 1; dcp < pcpubins[ src_bin_nr ].cpu_num; dcp++ ) {
        // get the distance information from the cpu at dcp to its node level agent cpu
        int dcp_dist = get_distance( node, pcpubins[ src_bin_nr ].cpu[ 0 ], pcpubins[ src_bin_nr ].cpu[ dcp ] );
        if( dcp_dist < 0 ) {
            continue;
        }

        for( int tcp = 1; tcp < pcpubins[ dest_bin_nr ].cpu_num; tcp++ ) {
            // get the distance information from the cpu at tcp to its node level agent cpu
            int tcp_dist = get_distance( node, pcpubins[ dest_bin_nr ].cpu[ 0 ], pcpubins[ dest_bin_nr ].cpu[ tcp ] );
            if( tcp_dist < 0 ) {
                continue;
            }

            // source bin: src_bin_nr,  source cpu: dcp,  dist to nodecpu: dcp_dist
            // dest bin: dest_bin_nr,   dest cpu: tcp,    dist to nodecpu: tcp_dist

            int dcp_dist_new = get_distance( node, pcpubins[ src_bin_nr ].cpu[ 0 ], pcpubins[ dest_bin_nr ].cpu[ tcp ] );
            int tcp_dist_new = get_distance( node, pcpubins[ dest_bin_nr ].cpu[ 0 ], pcpubins[ src_bin_nr ].cpu[ dcp ] );

            if( dcp_dist_new < 0 ) {
                continue;
            }
            if( tcp_dist_new < 0 ) {
                continue;
            }

            bool swap = false;

            // see if the swapping would improve the measure
            if( ( ( pcpubins[ dest_bin_nr ].active ) && ( dcp_dist + tcp_dist > dcp_dist_new + tcp_dist_new ) ) ||
                ( ( !pcpubins[ dest_bin_nr ].active ) && ( dcp_dist > dcp_dist_new ) ) ) {
                if( ( float )rand() / RAND_MAX < pprob ) {
                    swap = true;
                }
//				swap_performed=true;	// might not have been performed, but at least improvement detected
            }
            else {
                if( ( float )rand() / RAND_MAX < nprob ) {
                    swap = true;
                }
            }
            if( swap ) {
/*				psc_dbgmsg(1,"swapping %d:%d:%d(%d->%d) with [active: %x] %d:%d:%d(%d->%d)\n",
                                        src_bin_nr,dcp,pcpubins[src_bin_nr].cpu[dcp],dcp_dist,dcp_dist_new,
                                        pcpubins[dest_bin_nr].active,
                                        dest_bin_nr,tcp,pcpubins[dest_bin_nr].cpu[tcp],tcp_dist,tcp_dist_new); */
                int tmpcpu = pcpubins[ src_bin_nr ].cpu[ dcp ];
                pcpubins[ src_bin_nr ].cpu[ dcp ]  = pcpubins[ dest_bin_nr ].cpu[ tcp ];
                pcpubins[ dest_bin_nr ].cpu[ tcp ] = tmpcpu;
                swap_performed                     = true;
            }
        } // end for tcp
    }     // end for dcp
    return swap_performed;
}

bool BinStorage::opt_pcpu_position_rand( int node,
                                         int src_bin_nr,
                                         int dest_bin_nr,
                                         int count ) {
    if( node >= num_nodes ) {
        return false;
    }
    bininfo_rec* pcpubins       = &fullcpubins[ node * bins_per_node ];
    bool         swap_performed = false;

    for( int ct = 0; ct < count; ct++ ) {
        int dcp = ( int )( ( ( float )rand() * ( pcpubins[ src_bin_nr ].cpu_num - 1 ) ) / RAND_MAX );
        if( dcp >= pcpubins[ src_bin_nr ].cpu_num ) {
            dcp = pcpubins[ src_bin_nr ].cpu_num - 1;
        }
        int tcp = ( int )( ( ( float )rand() * ( pcpubins[ dest_bin_nr ].cpu_num - 1 ) ) / RAND_MAX );
        if( tcp >= pcpubins[ dest_bin_nr ].cpu_num ) {
            tcp = pcpubins[ dest_bin_nr ].cpu_num - 1;
        }

        // get old measure
        int smeasure_org = opt_get_measure( node, src_bin_nr );
        int dmeasure_org = opt_get_measure( node, dest_bin_nr );

        // swap cpu numbers
        int srccpunum  = pcpubins[ src_bin_nr ].cpu[ dcp ];
        int destcpunum = pcpubins[ dest_bin_nr ].cpu[ tcp ];
        pcpubins[ src_bin_nr ].cpu[ dcp ]  = destcpunum;
        pcpubins[ dest_bin_nr ].cpu[ tcp ] = srccpunum;

        // get new measure
        int smeasure_swp = opt_get_measure( node, src_bin_nr );
        int dmeasure_swp = opt_get_measure( node, dest_bin_nr );

        if( ( ( pcpubins[ dest_bin_nr ].active ) && ( smeasure_org + dmeasure_org > smeasure_swp + dmeasure_swp ) ) ||
            ( ( !pcpubins[ dest_bin_nr ].active ) && ( smeasure_org > smeasure_swp ) ) ) {
            swap_performed = true;
        }
        else {
            int s_newna = opt_nodeagent_get_position( node, src_bin_nr );
            int d_newna = opt_nodeagent_get_position( node, dest_bin_nr );

            // temporarily swap the node agent position
            int nnsrccpunum = pcpubins[ src_bin_nr ].cpu[ 0 ];
            int ncsrccpunum = pcpubins[ src_bin_nr ].cpu[ s_newna ];
            pcpubins[ src_bin_nr ].cpu[ 0 ]       = ncsrccpunum;
            pcpubins[ src_bin_nr ].cpu[ s_newna ] = nnsrccpunum;

            int nndestcpunum = pcpubins[ dest_bin_nr ].cpu[ 0 ];
            int ncdestcpunum = pcpubins[ dest_bin_nr ].cpu[ d_newna ];
            pcpubins[ dest_bin_nr ].cpu[ 0 ]       = ncdestcpunum;
            pcpubins[ dest_bin_nr ].cpu[ d_newna ] = nndestcpunum;

            smeasure_swp = opt_get_measure( node, src_bin_nr );
            dmeasure_swp = opt_get_measure( node, dest_bin_nr );

            if( ( ( pcpubins[ dest_bin_nr ].active ) && ( smeasure_org + dmeasure_org > smeasure_swp + dmeasure_swp ) ) ||
                ( ( !pcpubins[ dest_bin_nr ].active ) && ( smeasure_org > smeasure_swp ) ) ) {
                swap_performed = true;
            }
            else {
                // swap back everything

                pcpubins[ src_bin_nr ].cpu[ 0 ]       = nnsrccpunum;
                pcpubins[ src_bin_nr ].cpu[ s_newna ] = ncsrccpunum;

                pcpubins[ dest_bin_nr ].cpu[ 0 ]       = nndestcpunum;
                pcpubins[ dest_bin_nr ].cpu[ d_newna ] = ncdestcpunum;

                pcpubins[ src_bin_nr ].cpu[ dcp ]  = srccpunum;
                pcpubins[ dest_bin_nr ].cpu[ tcp ] = destcpunum;
            }
        }
    }

    return swap_performed;
}

void BinStorage::optimize_bins( int node ) {
    // optimize logical cluster

    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
    bininfo_rec* pcpubins  = &fullcpubins[ node * bins_per_node ];

    int dct;
    // find optimal node agent placement
    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        psc_dbgmsg( 5, "cluster %d, old distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );

        opt_nodeagent_position( node, dct );
    }

    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        psc_dbgmsg( 5, "cluster %d, new distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );
    }

    /*
       Now iterate over all disjunct cluster pairs (dct:tct) and over their cpus (dcp and tcp).
       Each cpu pair is then swapped and the distance measure evaluated. If the total
       measure decreases by this, the swap is performed.
       This ensures that the overall measure only decreases during the process,
       leading to a better distribution of the cpus over the clusters.
     */
    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        for( int tct = dct + 1; tct < fnum_of_cpubins[ node ]; tct++ ) {
            opt_pcpu_position( node, dct, tct );
        } // end for tct
    }     // end for dct

    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        for( int tct = fnum_of_cpubins[ node ] - 1; tct >= dct + 1; tct-- ) {
            opt_pcpu_position( node, dct, tct );
        }         // end for tct

        opt_nodeagent_position( node, dct );

        psc_dbgmsg( 5, "bin %d, opt distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );
    }     // end for dct
}

void BinStorage::optimize_bins_siman( int node ) {
    // optimize logical cluster

    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
    bininfo_rec* pcpubins  = &fullcpubins[ node * bins_per_node ];

    int dct;
    // find optimal node agent placement
    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        psc_dbgmsg( 5, "bin %d, old distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );

        opt_nodeagent_position( node, dct );
    }

    for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        psc_dbgmsg( 5, "bin %d, new distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );
    }

    float cprob       = 0.8;
    float cprob_nextf = 3.0;
    float cprob_negf  = 0.2;

    int  cstep    = 0;
    bool finished = false;
    while( !finished ) {
        finished = true;
        cstep++;
        for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
            if( !pcpubins[ dct ].active ) {
                continue;
            }
            for( int tct = 0; tct < fnum_of_cpubins[ node ]; tct++ ) {
                if( dct == tct ) {
                    continue;
                }
                if( opt_pcpu_position_siman( node, dct, tct, cprob, ( 1.0 - cprob ) * cprob_negf ) ) {
                    finished = false;
                }

                if( ( float )rand() / RAND_MAX < 1.0 - cprob ) {
                    // now update the node level agent placement for both bins
                    if( opt_nodeagent_position_siman( node, dct, cprob, ( 1.0 - cprob ) * cprob_negf ) ) {
                        finished = false;
                    }
                    if( pcpubins[ tct ].active ) {
                        if( opt_nodeagent_position( node, tct ) ) {
                            finished = false;
                        }
//						if (opt_nodeagent_position_siman(node, tct, cprob, (1.0-cprob)*cprob_negf)) finished=false;
                    }
                }
            }             // end for tct

            opt_nodeagent_position( node, dct );
        }         // end for dct

        psc_dbgmsg( 5, "step %d, prob %.2f, nprob %.2f\n", cstep, cprob, ( 1.0 - cprob ) * cprob_negf );
        for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
            psc_dbgmsg( 5, "   bin %d, opt distance sum: %d\n",
                        dct, opt_get_measure( node, dct ) );
        }

        // update parameters
        cprob = cprob + ( ( 1.0 - cprob ) / cprob_nextf );
        if( cprob > 0.99 ) {
            break;
        }

        if( cstep < 5 ) {
            finished = false;
        }
    }

    optimize_bins( node );
}

void BinStorage::optimize_bins_rand( int node ) {
    // optimize logical cluster

    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
    bininfo_rec* pcpubins  = &fullcpubins[ node * bins_per_node ];

    for( int dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
        if( !pcpubins[ dct ].active ) {
            continue;
        }
        psc_dbgmsg( 1, "bin %d, old distance sum: %d\n",
                    dct, opt_get_measure( node, dct ) );
    }

    int  cstep    = 0;
    bool finished = false;
    int  finct    = 8;
    while( !finished ) {
        finished = true;
        cstep++;
        int ct;
        int dct = ( int )( ( ( float )rand() * ( fnum_of_cpubins[ node ] - 1 ) ) / RAND_MAX );
        for( ct = 0; ct < 32; ct++ ) {
            if( pcpubins[ dct ].active ) {
                break;
            }
            dct = ( int )( ( ( float )rand() * ( fnum_of_cpubins[ node ] - 1 ) ) / RAND_MAX );
        }
        int tct = ( int )( ( ( float )rand() * ( fnum_of_cpubins[ node ] - 1 ) ) / RAND_MAX );
        for( ct = 0; ct < 32; ct++ ) {
            if( tct != dct ) {
                break;
            }
            tct = ( int )( ( ( float )rand() * ( fnum_of_cpubins[ node ] - 1 ) ) / RAND_MAX );
        }
        if( dct != tct ) {
            if( opt_pcpu_position_rand( node, dct, tct, 8 ) ) {
                finished = false;
            }
            // now update the node level agent placement for both bins
/*			if (opt_nodeagent_position_rand(node, dct, cprob, (1.0-cprob)*cprob_negf)) finished=false;
                        if (pcpubins[tct].active) {
                                if (opt_nodeagent_position_rand(node, tct, cprob, (1.0-cprob)*cprob_negf)) finished=false;
                        } */
        }

        psc_dbgmsg( 5, "step %d\n", cstep );
        for( dct = 0; dct < fnum_of_cpubins[ node ]; dct++ ) {
            psc_dbgmsg( 5, "   bin %d, opt distance sum: %d\n",
                        dct, opt_get_measure( node, dct ) );
        }

        if( !finished ) {
            finct = 8;
        }
        else {
            finct--;
            if( finct > 0 ) {
                finished = false;
            }
        }
    }

    optimize_bins( node );
}

void BinStorage::create_placement_file( int   node,
                                        char* placementFileName,
                                        int   ompnumthreads,
                                        int   total_procs ) {
    if( node >= num_nodes ) {
        return;
    }
    cpuinfo_rec* cpusetmap     = &fullcpusetmap[ node * cpus_per_node ];
    bininfo_rec* pcpubins      = &fullcpubins[ node * bins_per_node ];
    FILE*        placementFile = fopen( placementFileName, "w" );

    psc_dbgmsg( 5, "Creating placement file %s\n", placementFileName );
    char tmpb[ 2000 ];
    *tmpb = 0;

    int cbin_d  = 0;  //current bin
    int cproc_d = 1;  //current processor in bin. Assign 1 to skip first for analysis agent.
    for( int dct = 0; dct < total_procs; dct = dct + ompnumthreads ) {
        psc_dbgmsg( 5, "%d. MPI process in bin %d on cpu %d (rel:%d)\n", dct / ompnumthreads, cbin_d, pcpubins[ cbin_d ].cpu[ cproc_d ], cpusetmap[ pcpubins[ cbin_d ].cpu[ cproc_d ] ].relcpu );
        // see if this bin is large enough (nodeagent+ompthreads)
        while( pcpubins[ cbin_d ].cpu_num < 1 + ompnumthreads ) {
            //current bin too small to hold a process with its threads
            //goto next bin
            cbin_d++;
            cproc_d = 1;
            if( cbin_d > fnum_of_cpubins[ node ] ) {
                psc_dbgmsg( 5, "not enough well-sized clusters available!\n" );
                exit( 1 );
                break;
            }
        }

        //Store relativecpu of first threads for fork placement string
        if( dct != 0 ) {
            sprintf( tmpb, "%s,", tmpb );
        }
        sprintf( tmpb, "%s%d", tmpb, cpusetmap[ pcpubins[ cbin_d ].cpu[ cproc_d ] ].relcpu );


        if( ompnumthreads > 1 ) {
            //Generate statement for threads of this MPI process
            //oncpu is the relcpu of the MPI process and of thread 0
            fprintf( placementFile, "thread oncpu=%d cpu=", cpusetmap[ pcpubins[ cbin_d ].cpu[ cproc_d ] ].relcpu );
            for( int thread = 2; thread <= ompnumthreads; thread++ ) {
                cproc_d++;
                fprintf( placementFile, "%d", cpusetmap[ pcpubins[ cbin_d ].cpu[ cproc_d ] ].relcpu );
                psc_dbgmsg( 5, "CPU abs %d -> rel %d\n", pcpubins[ cbin_d ].cpu[ cproc_d ], cpusetmap[ pcpubins[ cbin_d ].cpu[ cproc_d ] ].relcpu );
                if( thread < ompnumthreads ) {
                    fprintf( placementFile, "," );
                }
            }
            fprintf( placementFile, " exact\n" );
        }
        cproc_d++;
        if( cproc_d >= pcpubins[ cbin_d ].cpu_num ) {
            cbin_d++;
            cproc_d = 1;
        }
    }
    fprintf( placementFile, "fork cpu=%s exact\n", tmpb );
    fclose( placementFile );
}


PBSLevelInfo* BinStorage::BinsToAgentHierarchy( int          numberOfNodes,
                                                std::string* pbsnodelist_string,
                                                int          ompnumthreads,
                                                int          mpinumprocs,
                                                int*         idmap,
                                                int*         pbsnodelist_distributed ) {
    char             buf[ 1000 ];
    PBSLevelInfo*    level, * masterLevel;
    PBSAgentDetails* agent;
    int              nextPort = fe->get_local_port() + 1;
    char             hostname[ 100 ];

    gethostname( hostname, 100 );

    if( ompnumthreads == 0 ) {
        ompnumthreads = 1;
    }
    psc_dbgmsg( 1, "Computing agent hierarchy for %d processes each with %d threads!\n", mpinumprocs, ompnumthreads );
    masterLevel = ( PBSLevelInfo* )calloc( 1, sizeof( PBSLevelInfo ) );
    level       = masterLevel;
    agent       = ( PBSAgentDetails* )calloc( 1, sizeof( PBSAgentDetails ) );
    sprintf( agent->port, "%d", nextPort );
    nextPort++;
    sprintf( agent->host, "%s", pbsnodelist_string[ 0 ].c_str() );
    sprintf( agent->parent, "%s[%d]", fe->get_local_tag().c_str(), fe->own_id() );

    sprintf( buf, "fe[%d]:0", fe->own_id() );
    strcat( agent->tag, buf );
    cpuinfo_rec* cpusetmap = &fullcpusetmap[ 0 ];
    bininfo_rec* pcpubins  = &fullcpubins[ 0 ];
    sprintf( buf, "%d", cpusetmap[ pcpubins[ 0 ].cpu[ 0 ] ].relcpu );
    strcat( agent->relCPU, buf );
    level->agents         = agent;
    level->numberOfAgents = 1;

    //if multiple nodes exist
    //for each node one high-level agent on next level
    if( numberOfNodes > 1 ) {
        level->prevLevel            = ( PBSLevelInfo* )calloc( 1, sizeof( PBSLevelInfo ) );
        level->prevLevel->nextLevel = level;
        level                       = level->prevLevel;
        PBSAgentDetails* parentAgent = level->nextLevel->agents;
        stringstream     children;
        for( int node = 0; node < numberOfNodes; node++ ) {
            PBSAgentDetails* lastAgent;
            cpuinfo_rec*     cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
            bininfo_rec*     pcpubins  = &fullcpubins[ node * bins_per_node ];

            agent = ( PBSAgentDetails* )calloc( 1, sizeof( PBSAgentDetails ) );
            sprintf( agent->port, "%d", nextPort );
            nextPort++;
            sprintf( agent->host, "%s", pbsnodelist_string[ node ].c_str() );
            sprintf( buf, "%s:%d", parentAgent->tag, node );
            strcat( agent->tag, buf );
            sprintf( buf, "%s", parentAgent->tag );
            strcat( agent->parent, buf );
            sprintf( buf, "%d", cpusetmap[ pcpubins[ 0 ].cpu[ 0 ] ].relcpu );
            strcat( agent->relCPU, buf );

            if( node == 0 ) {
                level->agents = agent;
                children << agent->tag;
                level->numberOfAgents = 1;
                lastAgent             = agent;
            }
            else {
                lastAgent->nextAgent = agent;
                children << "," << agent->tag;
                level->numberOfAgents++;
                lastAgent = agent;
            }
        }
        string help = children.str();
        strcat( parentAgent->children, help.c_str() );
    }

    //next level has already all analysis agents
    //ToDo: count processes, record IDs
    level->prevLevel            = ( PBSLevelInfo* )calloc( 1, sizeof( PBSLevelInfo ) );
    level->prevLevel->nextLevel = level;
    level                       = level->prevLevel;
    PBSAgentDetails* parentAgent = level->nextLevel->agents;

    int rank = 0;
    for( int node = 0; node < numberOfNodes; node++ ) {
        PBSAgentDetails* lastAgent;
        cpuinfo_rec*     cpusetmap = &fullcpusetmap[ node * cpus_per_node ];
        bininfo_rec*     pcpubins  = &fullcpubins[ node * bins_per_node ];
        stringstream     children;
        int              numberOfRanksInNode = 0;

        for( int bin = 0; bin < fnum_of_cpubins[ node ]; bin++ ) {
            stringstream ids;

            agent = ( PBSAgentDetails* )calloc( 1, sizeof( PBSAgentDetails ) );
            sprintf( agent->port, "%d", nextPort );
            nextPort++;
            sprintf( agent->host, "%s", pbsnodelist_string[ node ].c_str() );
            sprintf( buf, "%s:%d", parentAgent->tag, bin );
            strcat( agent->tag, buf );
            sprintf( buf, "%s", parentAgent->tag );
            strcat( agent->parent, buf );
            sprintf( buf, "%d", cpusetmap[ pcpubins[ bin ].cpu[ 0 ] ].relcpu );
            strcat( agent->relCPU, buf );

            psc_dbgmsg( 5, "Building id list for node %d with %d cpus in bin %d\n", node, pcpubins[ bin ].cpu_num, bin );
            for( int cpu = 1; cpu < pcpubins[ bin ].cpu_num; cpu++ ) {
                if( numberOfRanksInNode * ompnumthreads == pbsnodelist_distributed[ node ] ) {
                    break;                                                                                   //Stop if precalculated number of MPI processes in that node has been reached.
                }
                if( ( cpu - 1 ) % ompnumthreads == 0 ) {
                    //next mpi rank
                    if( cpu > 1 ) {
                        ids << ",";
                    }
                    ids << idmap[ rank ];
                    rank++;
                    numberOfRanksInNode++;
                }
            }

            if( !level->agents ) {
                level->agents         = agent;
                level->numberOfAgents = 1;
                lastAgent             = agent;
            }
            else {
                lastAgent->nextAgent = agent;
                level->numberOfAgents++;
                lastAgent = agent;
            }
            if( bin == 0 ) {
                children << agent->tag;
            }
            else {
                children << "," << agent->tag;
            }
            string help = ids.str();
            strcat( agent->applIds, help.c_str() );
            if( numberOfRanksInNode * ompnumthreads == pbsnodelist_distributed[ node ] ) {
                break;                                                                               //Stop going over bins because all MPI processes of that node have been placed
            }
        }
        string help = children.str();
        strcat( parentAgent->children, help.c_str() );
        parentAgent = parentAgent->nextAgent;
        if( rank == mpinumprocs ) {
            break;                            //All MPI processes have been assigned
        }
    }

    psc_dbgmsg( 1, "Agent hierarchy done!\n" );
    return masterLevel;
}
