/**
   @file    timing.c
   @brief   Periscope timing routines
   @author  Karl Fuerlinger
   @verbatim
    Revision:       $Revision$
    Revision date:  $Date$
    Committed by:   $Author$

    This file is part of the Periscope performance measurement tool.
    See http://www.lrr.in.tum.de/periscope for details.

    Copyright (c) 2005-2011, Technische Universitaet Muenchen, Germany
    See the COPYING file in the base directory of the package for details.
   @endverbatim
 */

#include "timing.h"
#include "psctypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


double psc_start_time;

double psc_wall_time( void ) {
    double current_time;

    PSC_WTIME( current_time );
    return current_time - psc_start_time;
}

void psc_init_start_time( void ) {
    PSC_WTIME( psc_start_time );
}

psc_ui8 rdtsc( void ) {
#if defined __GNUC__
#ifdef i386
    long long a;
    asm volatile ( "rdtsc" : "=A" ( a ) );
    return a;
#else /* gcc-IA64 version */
      /*
         unsigned long result;
         __asm__ __volatile__("mov %0=ar.itc" : "=r"(result) :: "memory");
         while (__builtin_expect ((int) result == -1, 0))
         __asm__ __volatile__("mov %0=ar.itc" : "=r"(result) :: "memory");
         return result;
       */
#endif

    /* Microsoft and Intel Windows compilers */
#elif defined _M_IX86
    __asm( "rdtsc" );
#elif defined _M_AMD64
    return __rdtsc();
#elif defined _M_IA64
#if defined __INTEL_COMPILER
#include <ia64intrin.h>
#endif
    return __getReg( 3116 );
#else
        #ifdef __p575
    return 0;
        #else
                #error need rdtsc function for this build
        #endif
#endif
    return 0;
}
